@@ -15,7 +15,7 @@ fn sum_slice() -> u32 {
 }
 
 fn sum_zerovec() -> u32 {
-    ZeroVec::<u32>::try_from_bytes(black_box(TEST_BUFFER_LE))
+    ZeroVec::<u32>::parse_byte_slice(black_box(TEST_BUFFER_LE))
         .unwrap()
         .iter()
         .sum::<u32>()
@@ -26,7 +26,7 @@ fn binarysearch_slice() -> Result<usize, usize> {
 }
 
 fn binarysearch_zerovec() -> Result<usize, usize> {
-    ZeroVec::<u32>::try_from_bytes(black_box(TEST_BUFFER_LE))
+    ZeroVec::<u32>::parse_byte_slice(black_box(TEST_BUFFER_LE))
         .unwrap()
         .binary_search(&0x0c0d0c)
 }
