@@ -7,14 +7,17 @@
 use crate::ule::AsULE;
 use crate::ZeroVec;
 use core::cmp::Ordering;
+use core::fmt;
 
+mod borrowed;
 mod kv;
 #[cfg(feature = "serde")]
 mod serde;
 mod vecs;
 
+pub use borrowed::ZeroMapBorrowed;
 pub use kv::ZeroMapKV;
-pub use vecs::ZeroVecLike;
+pub use vecs::{MutableZeroVecLike, ZeroVecLike};
 
 /// A zero-copy map datastructure, built on sorted binary-searchable [`ZeroVec`]
 /// and [`VarZeroVec`].
@@ -64,10 +67,7 @@ where
     V: ?Sized,
 {
     fn default() -> Self {
-        Self {
-            keys: K::Container::new(),
-            values: V::Container::new(),
-        }
+        Self::new()
     }
 }
 
@@ -78,9 +78,21 @@ where
     K: ?Sized,
     V: ?Sized,
 {
-    /// Construct a new [`ZeroMap`]
+    /// Creates a new, empty `ZeroMap<K, V>`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use zerovec::ZeroMap;
+    ///
+    /// let zm: ZeroMap<u16, str> = ZeroMap::new();
+    /// assert!(zm.is_empty());
+    /// ```
     pub fn new() -> Self {
-        Self::default()
+        Self {
+            keys: K::Container::new(),
+            values: V::Container::new(),
+        }
     }
 
     /// Construct a new [`ZeroMap`] with a given capacity
@@ -91,6 +103,14 @@ where
         }
     }
 
+    /// Obtain a borrowed version of this map
+    pub fn as_borrowed(&'a self) -> ZeroMapBorrowed<'a, K, V> {
+        ZeroMapBorrowed {
+            keys: self.keys.as_borrowed(),
+            values: self.values.as_borrowed(),
+        }
+    }
+
     /// The number of elements in the [`ZeroMap`]
     pub fn len(&self) -> usize {
         self.values.len()
@@ -127,7 +147,7 @@ where
     /// assert_eq!(map.get(&1), Some("one"));
     /// assert_eq!(map.get(&3), None);
     /// ```
-    pub fn get(&self, key: &K::NeedleType) -> Option<&V::GetType> {
+    pub fn get(&self, key: &K) -> Option<&V::GetType> {
         let index = self.keys.binary_search(key).ok()?;
         self.values.get(index)
     }
@@ -143,7 +163,7 @@ where
     /// assert_eq!(map.contains_key(&1), true);
     /// assert_eq!(map.contains_key(&3), false);
     /// ```
-    pub fn contains_key(&self, key: &K::NeedleType) -> bool {
+    pub fn contains_key(&self, key: &K) -> bool {
         self.keys.binary_search(key).is_ok()
     }
 
@@ -159,8 +179,7 @@ where
     /// assert_eq!(map.get(&3), None);
     /// ```
     pub fn insert(&mut self, key: &K, value: &V) -> Option<V::OwnedType> {
-        let key_needle = key.as_needle();
-        match self.keys.binary_search(key_needle) {
+        match self.keys.binary_search(key) {
             Ok(index) => Some(self.values.replace(index, value)),
             Err(index) => {
                 self.keys.insert(index, key);
@@ -181,7 +200,7 @@ where
     /// assert_eq!(map.remove(&1), Some("one".to_owned().into_boxed_str()));
     /// assert_eq!(map.get(&1), None);
     /// ```
-    pub fn remove(&mut self, key: &K::NeedleType) -> Option<V::OwnedType> {
+    pub fn remove(&mut self, key: &K) -> Option<V::OwnedType> {
         let idx = self.keys.binary_search(key).ok()?;
         self.keys.remove(idx);
         Some(self.values.remove(idx))
@@ -215,7 +234,7 @@ where
     pub fn try_append<'b>(&mut self, key: &'b K, value: &'b V) -> Option<(&'b K, &'b V)> {
         if self.keys.len() != 0 {
             if let Some(last) = self.keys.get(self.keys.len() - 1) {
-                if key.cmp_get(last) != Ordering::Greater {
+                if K::Container::t_cmp_get(key, last) != Ordering::Greater {
                     return Some((key, value));
                 }
             }
@@ -252,12 +271,12 @@ where
 
 impl<'a, K, V> ZeroMap<'a, K, V>
 where
-    K: ZeroMapKV<'a>,
-    V: ZeroMapKV<'a, Container = ZeroVec<'a, V>>,
+    K: ZeroMapKV<'a> + ?Sized,
+    V: ZeroMapKV<'a, Container = ZeroVec<'a, V>> + ?Sized,
     V: AsULE + Copy,
 {
     /// For cases when `V` is fixed-size, obtain a direct copy of `V` instead of `V::ULE`
-    pub fn get_copied(&self, key: &K::NeedleType) -> Option<V> {
+    pub fn get_copied(&self, key: &K) -> Option<V> {
         let index = self.keys.binary_search(key).ok()?;
         ZeroVec::get(&self.values, index)
     }
@@ -278,8 +297,8 @@ where
 
 impl<'a, K, V> ZeroMap<'a, K, V>
 where
-    K: ZeroMapKV<'a, Container = ZeroVec<'a, K>>,
-    V: ZeroMapKV<'a, Container = ZeroVec<'a, V>>,
+    K: ZeroMapKV<'a, Container = ZeroVec<'a, K>> + ?Sized,
+    V: ZeroMapKV<'a, Container = ZeroVec<'a, V>> + ?Sized,
     K: AsULE + Copy,
     V: AsULE + Copy,
 {
@@ -297,3 +316,63 @@ where
         })
     }
 }
+
+impl<'a, K, V> From<ZeroMapBorrowed<'a, K, V>> for ZeroMap<'a, K, V>
+where
+    K: ZeroMapKV<'a>,
+    V: ZeroMapKV<'a>,
+    K: ?Sized,
+    V: ?Sized,
+{
+    fn from(other: ZeroMapBorrowed<'a, K, V>) -> Self {
+        Self {
+            keys: K::Container::from_borrowed(other.keys),
+            values: V::Container::from_borrowed(other.values),
+        }
+    }
+}
+
+// We can't use the default PartialEq because ZeroMap is invariant
+// so otherwise rustc will not automatically allow you to compare ZeroMaps
+// with different lifetimes
+impl<'a, 'b, K, V> PartialEq<ZeroMap<'b, K, V>> for ZeroMap<'a, K, V>
+where
+    K: for<'c> ZeroMapKV<'c> + ?Sized,
+    V: for<'c> ZeroMapKV<'c> + ?Sized,
+    <K as ZeroMapKV<'a>>::Container: PartialEq<<K as ZeroMapKV<'b>>::Container>,
+    <V as ZeroMapKV<'a>>::Container: PartialEq<<V as ZeroMapKV<'b>>::Container>,
+{
+    fn eq(&self, other: &ZeroMap<'b, K, V>) -> bool {
+        self.keys.eq(&other.keys) && self.values.eq(&other.values)
+    }
+}
+
+impl<'a, K, V> fmt::Debug for ZeroMap<'a, K, V>
+where
+    K: ZeroMapKV<'a> + ?Sized,
+    V: ZeroMapKV<'a> + ?Sized,
+    <K as ZeroMapKV<'a>>::Container: fmt::Debug,
+    <V as ZeroMapKV<'a>>::Container: fmt::Debug,
+{
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> Result<(), fmt::Error> {
+        f.debug_struct("ZeroMap")
+            .field("keys", &self.keys)
+            .field("values", &self.values)
+            .finish()
+    }
+}
+
+impl<'a, K, V> Clone for ZeroMap<'a, K, V>
+where
+    K: ZeroMapKV<'a> + ?Sized,
+    V: ZeroMapKV<'a> + ?Sized,
+    <K as ZeroMapKV<'a>>::Container: Clone,
+    <V as ZeroMapKV<'a>>::Container: Clone,
+{
+    fn clone(&self) -> Self {
+        Self {
+            keys: self.keys.clone(),
+            values: self.values.clone(),
+        }
+    }
+}
