@@ -2,8 +2,7 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-use super::{ZeroMap, ZeroMapKV, ZeroVecLike};
-use alloc::boxed::Box;
+use super::{MutableZeroVecLike, ZeroMap, ZeroMapBorrowed, ZeroMapKV, ZeroVecLike};
 use core::fmt;
 use core::marker::PhantomData;
 use serde::de::{self, MapAccess, Visitor};
@@ -19,8 +18,8 @@ where
     V: ?Sized,
     K::Container: Serialize,
     V::Container: Serialize,
-    K::SerializeType: Serialize,
-    V::SerializeType: Serialize,
+    K: Serialize,
+    V: Serialize,
 {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
@@ -29,8 +28,8 @@ where
         if serializer.is_human_readable() {
             let mut map = serializer.serialize_map(Some(self.len()))?;
             for (k, v) in self.iter() {
-                K::with_ser(k, |k| map.serialize_key(k))?;
-                V::with_ser(v, |v| map.serialize_value(v))?;
+                K::Container::t_with_ser(k, |k| map.serialize_key(k))?;
+                V::Container::t_with_ser(v, |v| map.serialize_value(v))?;
             }
             map.end()
         } else {
@@ -39,13 +38,33 @@ where
     }
 }
 
-/// Modified example from https://serde.rs/deserialize-map.html
-struct ZeroMapMapVisitor<K: ?Sized, V: ?Sized> {
-    #[allow(clippy::type_complexity)] // it's a marker type, complexity doesn't matter
-    marker: PhantomData<fn() -> (Box<K>, Box<V>)>,
+/// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
+impl<'a, K, V> Serialize for ZeroMapBorrowed<'a, K, V>
+where
+    K: ZeroMapKV<'a>,
+    V: ZeroMapKV<'a>,
+    K: ?Sized,
+    V: ?Sized,
+    K::Container: Serialize,
+    V::Container: Serialize,
+    K: Serialize,
+    V: Serialize,
+{
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        ZeroMap::<K, V>::from(*self).serialize(serializer)
+    }
 }
 
-impl<K: ?Sized, V: ?Sized> ZeroMapMapVisitor<K, V> {
+/// Modified example from https://serde.rs/deserialize-map.html
+struct ZeroMapMapVisitor<'a, K: ZeroMapKV<'a> + ?Sized, V: ZeroMapKV<'a> + ?Sized> {
+    #[allow(clippy::type_complexity)] // it's a marker type, complexity doesn't matter
+    marker: PhantomData<fn() -> (&'a K::OwnedType, &'a V::OwnedType)>,
+}
+
+impl<'a, K: ZeroMapKV<'a> + ?Sized, V: ZeroMapKV<'a> + ?Sized> ZeroMapMapVisitor<'a, K, V> {
     fn new() -> Self {
         ZeroMapMapVisitor {
             marker: PhantomData,
@@ -53,15 +72,15 @@ impl<K: ?Sized, V: ?Sized> ZeroMapMapVisitor<K, V> {
     }
 }
 
-impl<'de, K: ?Sized, V: ?Sized> Visitor<'de> for ZeroMapMapVisitor<K, V>
+impl<'a, 'de, K: ?Sized, V: ?Sized> Visitor<'de> for ZeroMapMapVisitor<'a, K, V>
 where
     K: Ord,
-    K: ZeroMapKV<'de>,
-    V: ZeroMapKV<'de>,
+    K: ZeroMapKV<'a>,
+    V: ZeroMapKV<'a>,
     K::OwnedType: Deserialize<'de>,
     V::OwnedType: Deserialize<'de>,
 {
-    type Value = ZeroMap<'de, K, V>;
+    type Value = ZeroMap<'a, K, V>;
 
     fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
         formatter.write_str("a map produced by ZeroMap")
@@ -80,7 +99,10 @@ where
             // If not sorted, return an error
             // a serialized map that came from another ZeroMap
             if map
-                .try_append(K::owned_as_self(&key), V::owned_as_self(&value))
+                .try_append(
+                    K::Container::owned_as_t(&key),
+                    V::Container::owned_as_t(&value),
+                )
                 .is_some()
             {
                 return Err(de::Error::custom(
@@ -94,22 +116,23 @@ where
 }
 
 /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
-impl<'de, K: ?Sized, V: ?Sized> Deserialize<'de> for ZeroMap<'de, K, V>
+impl<'de, 'a, K: ?Sized, V: ?Sized> Deserialize<'de> for ZeroMap<'a, K, V>
 where
     K: Ord,
     K::Container: Deserialize<'de>,
     V::Container: Deserialize<'de>,
-    K: ZeroMapKV<'de>,
-    V: ZeroMapKV<'de>,
+    K: ZeroMapKV<'a>,
+    V: ZeroMapKV<'a>,
     K::OwnedType: Deserialize<'de>,
     V::OwnedType: Deserialize<'de>,
+    'de: 'a,
 {
     fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
     where
         D: Deserializer<'de>,
     {
         if deserializer.is_human_readable() {
-            deserializer.deserialize_map(ZeroMapMapVisitor::new())
+            deserializer.deserialize_map(ZeroMapMapVisitor::<'a, K, V>::new())
         } else {
             let (keys, values): (K::Container, V::Container) =
                 Deserialize::deserialize(deserializer)?;
@@ -126,10 +149,64 @@ where
     }
 }
 
+// /// This impl can be made available by enabling the optional `serde` feature of the `zerovec` crate
+impl<'de, 'a, K: ?Sized, V: ?Sized> Deserialize<'de> for ZeroMapBorrowed<'a, K, V>
+where
+    K: Ord,
+    K::Container: Deserialize<'de>,
+    V::Container: Deserialize<'de>,
+    K: ZeroMapKV<'a>,
+    V: ZeroMapKV<'a>,
+    K::OwnedType: Deserialize<'de>,
+    V::OwnedType: Deserialize<'de>,
+    'de: 'a,
+{
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        if deserializer.is_human_readable() {
+            Err(de::Error::custom(
+                "ZeroMapBorrowed cannot be deserialized from human-readable formats",
+            ))
+        } else {
+            let deserialized: ZeroMap<'a, K, V> = ZeroMap::deserialize(deserializer)?;
+            let keys = if let Some(keys) = deserialized.keys.as_borrowed_inner() {
+                keys
+            } else {
+                return Err(de::Error::custom(
+                    "ZeroMapBorrowed can only deserialize in zero-copy ways",
+                ));
+            };
+            let values = if let Some(values) = deserialized.values.as_borrowed_inner() {
+                values
+            } else {
+                return Err(de::Error::custom(
+                    "ZeroMapBorrowed can only deserialize in zero-copy ways",
+                ));
+            };
+            Ok(Self { keys, values })
+        }
+    }
+}
+
 #[cfg(test)]
+#[allow(non_camel_case_types)]
 mod test {
     use super::super::*;
 
+    #[derive(::serde::Serialize, ::serde::Deserialize)]
+    struct DeriveTest_ZeroMap<'data> {
+        #[serde(borrow)]
+        _data: ZeroMap<'data, str, [u8]>,
+    }
+
+    #[derive(::serde::Serialize, ::serde::Deserialize)]
+    struct DeriveTest_ZeroMapBorrowed<'data> {
+        #[serde(borrow)]
+        _data: ZeroMapBorrowed<'data, str, [u8]>,
+    }
+
     const JSON_STR: &str = "{\"1\":\"uno\",\"2\":\"dos\",\"3\":\"tres\"}";
     const BINCODE_BYTES: &[u8] = &[
         12, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3, 0, 0, 0, 26, 0, 0, 0, 0, 0, 0, 0, 3, 0,
@@ -165,5 +242,12 @@ mod test {
             new_map.iter().collect::<Vec<_>>(),
             map.iter().collect::<Vec<_>>()
         );
+
+        let new_map: ZeroMapBorrowed<u32, str> =
+            bincode::deserialize(&bincode_bytes).expect("deserialize");
+        assert_eq!(
+            new_map.iter().collect::<Vec<_>>(),
+            map.iter().collect::<Vec<_>>()
+        );
     }
 }
